/**
 * PolicyDB.cpp: PolicyDB.h implementation
 * Copyright 2005 by Columbia University; all rights reserved
 */

#include "PolicyDB.h"

/*
 * PolicyDB::PolicyDB(char *presentity,char *database,char *machine,char *user,char *password)
 * This is a constructor. It initializes the DOM parser,rule object and PolicyQuery object(query).
 */

PolicyDB :: PolicyDB (char *presentity,char *database,char *machine,char *user,char *password) {
  
  try {
    XMLPlatformUtils::Initialize();
  }
  catch(const XMLException &toCatch) {
    cerr<<"Error due to initialization"<<endl;
    exit(1);
  }
  
  parser = new XercesDOMParser();
  parser->setValidationScheme(XercesDOMParser::Val_Never);
  parser->setDoNamespaces(true);   
  parser->setIncludeIgnorableWhitespace(false);
  
  errhandler = (ErrorHandler *)new HandlerBase();
  parser->setErrorHandler(errhandler);
  query = new PolicyQuery(database,machine,user,password);
  query->verify_presentity(presentity);
  policy_rule = new rule();	
  policy_rule->presentity = presentity;
  cout<<"Assigned presentity "<<policy_rule->presentity<<endl;	
  cout <<"Initializing all elements of rule"<<endl;	
  reset_rule();		   
  cout<<"Initialization successful\n";	
}/* PolicyDB::PolicyDB(char *presentity,,char *database,char *machine,char *user,char *password) */


/*
 * PolicyDB::~PolicyDB()
 * This is a destructor. It deletes the DOM parser,rule object,error handler and PolicyQuery object(query).
 */

PolicyDB :: ~PolicyDB() {
    
    delete parser;
    delete errhandler;
    delete policy_rule;
    delete query;
    XMLPlatformUtils::Terminate();
}/* PolicyDB::~PolicyDB() */


/*
 * PolicyDB::parsePolicyDocument(char *policydoc)
 * This function starts the parsing of the policy document
 * Returns true if successful else false.
 */

bool PolicyDB :: parsePolicyDocument(char *policydoc) {

    char* xmlText = (char*)calloc(strlen(policydoc) + 10, sizeof(char));
    strcpy(xmlText, policydoc);
    char* xmlBufId = "presence-id";
    MemBufInputSource* memsource = new MemBufInputSource((XMLByte* )xmlText, 
						     strlen(xmlText)+1, 
						     (XMLCh* )xmlBufId, 
						     false);
    try {
        parser->parse(*memsource);
    }
    catch (const XMLException& toCatch) {
        printf("Exception occurred while parsing\n");
        cerr<<"XMLException : "<<XMLString::transcode(toCatch.getMessage())<<endl;
        delete parser;
        delete errhandler;
	free(xmlText);
        return false;
    }
    catch (const DOMException& toCatch) {
        cerr<<"Exception occurred while parsing \n";
        delete parser;
        delete errhandler;
	free(xmlText);
        return false;
    }
    catch (const SAXException& toCatch) {
        cerr<<"SAXException : "<<XMLString::transcode(toCatch.getMessage())<<endl;
        delete parser;
        delete errhandler;
	free(xmlText);
        return false;
    }
    catch (...) {
        printf("Exception occurred while parsing\n");
        delete parser;
        delete errhandler;
	free(xmlText);
        return false;
    }

    DOMDocument *doc = parser->getDocument();
    DOMNode *cur_node = doc->getFirstChild();
    const XMLCh *common_policy_prefix = cur_node->lookupNamespacePrefix(XMLString::transcode("urn:ietf:params:xml:ns:common-policy"),false);
    if(common_policy_prefix == NULL) {
      cerr<<"Common policy namespace not found"<<endl;
      exit(1);
    }
    else {
      cout<<"Common policy namespace : "<<XMLString::transcode(common_policy_prefix)<<endl;	
    }
    if(strcmp(XMLString::transcode(cur_node->getLocalName()),"ruleset") == 0) {
      	cout<<"Processing "<<XMLString::transcode(cur_node->getNodeName())<<endl;	
      	processRuleset(cur_node,common_policy_prefix);
    }
    else {
      cerr<<"Ruleset not found"<<endl;
      free(xmlText);
      return false;
    }
    free(xmlText);
    return true;

}/* PolicyDB::parsePolicyDocument(char *policydoc) */


/* 
 * PolicyDB::processConditions(DOMNode *conditions,const XMLCh *common_policy_prefix)
 * This function parses the <conditions> and all its children of the policy document.
 * Inserts identity condition to the database.Sphere and Validity are stored in the rule object
 * for insertion later.
 */

void PolicyDB :: processConditions(DOMNode *conditions,const XMLCh *common_policy_prefix) {

    cout<<"Condition element is "<<XMLString::transcode(conditions->getNodeName())<<"\n"<<endl;
    DOMNode* conditionChild = conditions->getFirstChild();
       
    while(conditionChild !=NULL) {
        
     if(strcmp(XMLString::transcode(conditionChild->getNodeName()),"#text") == 0) {
         conditionChild = conditionChild->getNextSibling();
         continue;
     }
	
      cout<<"Element encountered : "<<XMLString::transcode(conditionChild->getNodeName())<<endl;	
   
     
      char* localvalue = XMLString::transcode(conditionChild->getLocalName());
      char* prefixvalue = XMLString::transcode(conditionChild->getPrefix());
      char* common_prefix = XMLString::transcode(common_policy_prefix);
      
      if((strcmp(localvalue,"identity")==0) && (strcmp(prefixvalue,common_prefix) == 0)){
	
	DOMNode *id = (DOMNode *)conditionChild ->getFirstChild();
	while (id != NULL) {
	  
	  if(strcmp(XMLString::transcode(id->getNodeName()),"#text") == 0) {
             id = id->getNextSibling();
             continue;
          }
	
          cout<<"Element encountered : "<<XMLString::transcode(id->getNodeName())<<endl;	
	  if (strcmp(XMLString::transcode(id ->getLocalName()),"one")==0){
	    
	    if(((DOMElement *)id)->hasAttribute(XMLString::transcode("id"))) {
	      policy_rule->one_id = XMLString::transcode(((DOMElement *)id)->getAttribute(XMLString::transcode("id")));

	      cout<<"One_id is "<<policy_rule->one_id<<"\n"<<endl;
	    }
	    
	    if(((DOMElement *)id)->hasAttribute(XMLString::transcode("scheme"))) {
	      policy_rule->one_scheme = XMLString::transcode(((DOMElement *)id)->getAttribute(XMLString::transcode("schema")));

	      cout<<"\nOne_scheme is"<<policy_rule->one_scheme<<"\n"<<endl;
	    }
                    
	    query->insert_ruleidentity((char *)policy_rule->presentity,(char *)policy_rule->rule_id,(char *)policy_rule->one_id, (char *)policy_rule->one_scheme,NULL,NULL,NULL,NULL);                        
	  
	  } 
	  else if (strcmp(XMLString::transcode(id ->getLocalName()),"many")==0) {
	    
	    DOMNode *except_child = id->getFirstChild();
	    // <many> has attribute domain
		
	    if (((DOMElement *)id)->hasAttribute(XMLString::transcode("domain"))) {
	      
	      policy_rule->many_domain = XMLString::transcode(((DOMElement *)id)->getAttribute(XMLString::transcode("domain")));
	      while(except_child !=NULL) {
                 		
                if(strcmp(XMLString::transcode(except_child->getNodeName()),"#text") == 0) {
                    except_child = except_child->getNextSibling();
                    continue;
                }
	
                cout<<"Element encountered : "<<XMLString::transcode(except_child->getNodeName())<<endl;	
		if (((DOMElement *)except_child)->hasAttribute(XMLString::transcode("id"))) {
		  policy_rule->except_id = XMLString::transcode(((DOMElement *)except_child)->getAttribute(XMLString::transcode("id")));
		
		  
		  query->insert_ruleidentity((char *)policy_rule->presentity,(char *)policy_rule->rule_id,NULL,NULL,(char *)policy_rule->except_id,NULL,NULL,(char *)policy_rule->many_domain);
		}
	 	except_child = except_child->getNextSibling();
		
	      }
	    }
	    
	    // <many> has no attributes 
	    else { 
	      while(except_child != NULL) { 
	      // <except> has attribute domain
	      
                if(strcmp(XMLString::transcode(except_child->getNodeName()),"#text") == 0) {
                    except_child = except_child->getNextSibling();
                    continue;
                }
	
                cout<<"Element encountered : "<<XMLString::transcode(((DOMElement *)except_child)->getNodeName())<<endl;	
	         
	        if (((DOMElement *)except_child)->hasAttribute(XMLString::transcode("domain"))) {
		    
		   policy_rule->except_domain = XMLString::transcode(((DOMElement *)except_child)->getAttribute(XMLString::transcode("domain")));
		  
                   // insert ruleid,except_domain
		   query->insert_ruleidentity((char *)policy_rule->presentity,(char *)policy_rule->rule_id,NULL,NULL,NULL,NULL,(char *)policy_rule->except_domain,NULL);
		    
                
	         }
	         // <except> has attribute id or id and schema
       	         else if (((DOMElement *)except_child)->hasAttribute(XMLString::transcode("id"))) {
		
		    policy_rule->except_id  = XMLString::transcode(((DOMElement *)except_child)->getAttribute(XMLString::transcode("id")));
		
		    if (((DOMElement *)except_child)->hasAttribute(XMLString::transcode("scheme"))) {
		  
		       policy_rule->except_scheme = XMLString::transcode(((DOMElement *)except_child)->getAttribute(XMLString::transcode("scheme")));
		      query->insert_ruleidentity((char *)policy_rule->presentity,(char *)policy_rule->rule_id,NULL,NULL,(char *)policy_rule->except_id,(char *)policy_rule->except_scheme,NULL,NULL);
		    }
		    else {
		  
		      query->insert_ruleidentity((char *)policy_rule->presentity,(char *)policy_rule->rule_id,NULL,NULL,(char *)policy_rule->except_id,NULL,NULL,NULL);
		    }
	         }	     
	      
	         except_child = except_child->getNextSibling();
	      }
	    }
            
	  }

	 id = id->getNextSibling();
	}        
       } 
       
       if((strcmp(localvalue,"sphere")==0) && (strcmp(prefixvalue,common_prefix) == 0)) {

	  if(((DOMElement *)conditionChild)->hasAttribute(XMLString::transcode("value"))) {
	    policy_rule->sphere = XMLString::transcode(((DOMElement *)conditionChild)->getAttribute(XMLString::transcode("value")));
	    cout<<"Sphere is "<<policy_rule->sphere<<"\n"<<endl;
	  }
        }

      
        
        // modify this
        if((strcmp(localvalue,"validity")==0) && (strcmp(prefixvalue,common_prefix) == 0)) {
	
            DOMNode* validity = conditionChild->getFirstChild();
            

            while(validity!=NULL) {
	        
                if(strcmp(XMLString::transcode(validity->getNodeName()),"#text") == 0) {
                    validity = validity->getNextSibling();
                    continue;
                }
	
                cout<<"Element encountered : "<<XMLString::transcode(validity->getNodeName())<<endl;	
                 
                if(strcmp(XMLString::transcode(validity->getLocalName()),"from")==0) {
                    policy_rule->validity_from = (char *)(convert_to_sqldatetime(XMLString::transcode(validity->getTextContent())).c_str());
                }
                else if(strcmp(XMLString::transcode(validity->getLocalName()),"until")==0) {
                   policy_rule->validity_to = (char *)(convert_to_sqldatetime(XMLString::transcode(validity->getTextContent())).c_str());
                }
                validity=validity->getNextSibling();
            }

        }
       
       conditionChild = conditionChild->getNextSibling();
       XMLString::release(&localvalue); 
       XMLString::release(&prefixvalue);
       XMLString::release(&common_prefix);
     
    }// end while        
    return;	
}/* PolicyDB::processConditions(DOMNode *conditions,const XMLCh *common_policy_prefix) */

/* PolicyDB :: processActions(DOMNode *actions)
 * This function parses the <actions> element and its children in the DOM of the policy document.
 * Stores the appropriate parsed values in the rule object.
 */
 
void PolicyDB :: processActions(DOMNode *actions) {

    cout<<"Action element is "<<XMLString::transcode(actions->getNodeName())<<"\n"<<endl;
    DOMNode* actionChild = actions->getFirstChild();
    
    while(actionChild !=NULL) {
      	
      if(strcmp(XMLString::transcode(actionChild->getNodeName()),"#text") == 0) {
         actionChild = actionChild->getNextSibling();
         continue;
      }
	
      cout<<"Element encountered : "<<XMLString::transcode(actionChild->getNodeName())<<endl;	
     
      char* localvalue = XMLString::transcode(actionChild->getLocalName());
      if(strcmp(localvalue,"sub-handling")==0){
	policy_rule->sub_handling = XMLString::transcode(actionChild->getTextContent());
	cout<<"Subscription is "<<policy_rule->sub_handling<<"\n"<<endl;
      }
     
      actionChild = actionChild->getNextSibling();
      XMLString::release(&localvalue);
     
   }
   return;
}/* PolicyDB :: processActions(DOMNode *actions) */


/* PolicyDB :: processTransformations(DOMNode *transformations)
 * This function parses the <transformations> element and its children in the DOM of the policy document.
 * Stores the appropriate parsed values in the rule object.
 */

void PolicyDB :: processTransformations(DOMNode *transformations){
    
  cout<<"Transformation element is "<<XMLString::transcode(transformations->getNodeName())<<"\n"<<endl;
  DOMNode* transformationChild = transformations->getFirstChild();

  while(transformationChild != NULL) {
    
    if(strcmp(XMLString::transcode(transformationChild->getNodeName()),"#text") == 0) {
        transformationChild = transformationChild->getNextSibling();
        continue;
    }
	
    cout<<"Element encountered : "<<XMLString::transcode(transformationChild->getNodeName())<<endl;	

    char* localvalue = XMLString::transcode(transformationChild->getLocalName());
    
    cout<<"Local value : "<<localvalue<<endl;
    if(strcmp(localvalue,"provide-services")==0){

	DOMNode* serviceChild = transformationChild->getFirstChild();
	while(serviceChild!=NULL) {
		  
          if(strcmp(XMLString::transcode(serviceChild->getNodeName()),"#text") == 0) {
               serviceChild = serviceChild->getNextSibling();
               continue;
          }
	
          cout<<"Element encountered : "<<XMLString::transcode(serviceChild->getNodeName())<<endl;	
	  
	  char* service = XMLString::transcode(serviceChild->getLocalName());
	  if(strcmp(service,"all-services")==0) {
	    policy_rule->all_services = "true";
	    cout<<"All-services is "<<XMLString::transcode(serviceChild->getNodeName())<<":"<<policy_rule->all_services<<"\n"<<endl;   
	  }
	  else {
	    processServicePermission(service,XMLString::transcode(serviceChild->getTextContent()));
	  }
	  
	  serviceChild=serviceChild->getNextSibling();
	  XMLString::release(&service);
	}
	
      }
    
      else if(strcmp(localvalue,"provide-devices")==0){
        cout<<"Reached here : provide-devices"<<endl;
	DOMNode* deviceChild = transformationChild->getFirstChild();
	while(deviceChild!=NULL) {
	  
           if(strcmp(XMLString::transcode(deviceChild->getNodeName()),"#text") == 0) {
               deviceChild = deviceChild->getNextSibling();
           continue;
           }
	
          cout<<"Element encountered : "<<XMLString::transcode(deviceChild->getNodeName())<<endl;	
	  char* device = XMLString::transcode(deviceChild->getLocalName());
          cout <<"Device : "<<device<<endl;
	  if(strcmp(device,"all-devices")==0) {
	    policy_rule->all_devices = "true";
	    cout<<"All-devices is "<<XMLString::transcode(deviceChild->getNodeName())<<":"<<policy_rule->all_devices<<"\n"<<endl;   
	  }
	  else {
	    processDevicePermission(device,XMLString::transcode(deviceChild->getTextContent()));
	  }
        
	  deviceChild=deviceChild->getNextSibling();
	  XMLString::release(&device);
	}
      } 
      
      else if(strcmp(localvalue,"provide-persons")==0){
	DOMNode* personChild = transformationChild->getFirstChild();
	while(personChild!=NULL) {
	  
           if(strcmp(XMLString::transcode(personChild->getNodeName()),"#text") == 0) {
               personChild = personChild->getNextSibling();
               continue;
           }
	
          cout<<"Element encountered : "<<XMLString::transcode(personChild->getNodeName())<<endl;	
	  char* person = XMLString::transcode(personChild->getNodeName());
	  if(strcmp(person,"all-persons")==0) {
	    policy_rule->all_persons = "true";
	    cout<<"All-persons is "<<XMLString::transcode(personChild->getNodeName())<<":"<<policy_rule->all_persons<<"\n"<<endl;
	  }
	  else {
	    processPersonPermission(person,XMLString::transcode(personChild->getNodeValue()));
	  }
        
	  personChild=personChild->getNextSibling();
	  XMLString::release(&person);
	}
      }
      
      if(strcmp(localvalue,"provide-activities")==0){
	
	policy_rule->provide_activities = XMLString::transcode(transformationChild->getTextContent());
	cout<<"Provide-activities is "<<policy_rule->provide_activities<<"\n"<<endl;
      }
      
      if(strcmp(localvalue,"provide-class")==0){
	
	policy_rule->provide_class = XMLString::transcode(transformationChild->getTextContent());
	cout<<"Provide-class is "<<policy_rule->provide_class<<"\n"<<endl;
      }

      if(strcmp(localvalue,"provide-device-id")==0){
      
	policy_rule->provide_device_id = XMLString::transcode(transformationChild->getTextContent());
	cout<<"Provide-device-id is "<<policy_rule->provide_device_id<<"\n"<<endl;
      }
      
      if(strcmp(localvalue,"provide-mood")==0){
      
	policy_rule->provide_mood = XMLString::transcode(transformationChild->getTextContent());
	cout<<"Provide-mood is "<<policy_rule->provide_mood<<"\n"<<endl;
      }
      
      if(strcmp(localvalue,"provide-place-is")==0){
      
	policy_rule->provide_place_is = XMLString::transcode(transformationChild->getTextContent());
	cout<<"Provide-place-is is "<<policy_rule->provide_place_is<<"\n"<<endl;
      }
    
      if(strcmp(localvalue,"provide-place-type")==0){
	
	policy_rule->provide_place_type = XMLString::transcode(transformationChild->getTextContent());
	cout<<"Provide-place-type is "<<policy_rule->provide_place_type<<"\n"<<endl;
      }
      
      if(strcmp(localvalue,"provide-privacy")==0){
	
	policy_rule->provide_privacy = XMLString::transcode(transformationChild->getTextContent());
	cout<<"Provide-privacy is "<<policy_rule->provide_privacy<<"\n"<<endl;
      }
      
      if(strcmp(localvalue,"provide-relationship")==0){
      
	policy_rule->provide_relationship = XMLString::transcode(transformationChild->getTextContent());
	cout<<"Provide-relationship is "<<policy_rule->provide_relationship<<"\n"<<endl;
      }

      if(strcmp(localvalue,"provide-status-icon")==0){
	
	policy_rule->provide_status_icon = XMLString::transcode(transformationChild->getTextContent());
	cout<<"Provide-status-icon is "<<policy_rule->provide_status_icon<<"\n"<<endl;
      }

      if(strcmp(localvalue,"provide-sphere")==0){
      
	policy_rule->provide_sphere = XMLString::transcode(transformationChild->getTextContent());
	cout<<"Provide-sphere is "<<policy_rule->provide_sphere<<"\n"<<endl;
      }
      
      if(strcmp(localvalue,"provide-time-offset")==0){
          
	policy_rule->provide_time_offset = XMLString::transcode(transformationChild->getTextContent());
	cout<<"Provide-time-offset is "<<policy_rule->provide_time_offset<<"\n"<<endl;
      }
      
      if(strcmp(localvalue,"provide-user-input")==0){
      
	policy_rule->provide_user_input = XMLString::transcode(transformationChild->getTextContent());
	cout<<"Provide-user-input is "<<policy_rule->provide_user_input<<"\n"<<endl;
      }

      if(strcmp(localvalue,"provide-note")==0){
	
	policy_rule->provide_note = XMLString::transcode(transformationChild->getTextContent());
	cout<<"Provide-note is "<<policy_rule->provide_note<<"\n"<<endl;
      }
    
      if(strcmp(localvalue,"provide-unknown-attribute")==0){
          
	policy_rule->provide_unknown_attribute = XMLString::transcode(transformationChild->getTextContent());
	if(((DOMElement *)transformationChild)->hasAttribute(XMLString::transcode("name"))) {
	  policy_rule->unknown_attribute_name = XMLString::transcode(((DOMElement *)transformationChild)->getAttribute(XMLString::transcode("name")));
	}
	cout<<"Provide-unknown-attribute is "<<policy_rule->provide_unknown_attribute<<"\n"<<endl;
	cout<<"Unknown-attribute-name is "<<policy_rule->unknown_attribute_name<<"\n"<<endl;
      }
      
      if(strcmp(localvalue,"provide-all-attributes")==0) {
	 //other permissions
	   policy_rule->provide_activities = "true";
	   policy_rule->provide_class = "true";
	   policy_rule->provide_device_id="true";
	   policy_rule->provide_mood = "true";
	   policy_rule->provide_place_is = "true";
	   policy_rule->provide_place_type = "true";
	   policy_rule->provide_privacy = "true";
	   policy_rule->provide_relationship = "true";
	   policy_rule->provide_status_icon = "true";
	   policy_rule->provide_sphere = "true";
	   policy_rule->provide_time_offset = "true";
           policy_rule->provide_user_input = "full";
           policy_rule->provide_note = "true";
           policy_rule->provide_unknown_attribute = "true";
           policy_rule->unknown_attribute_name = NULL;
	   cout<<"Permission is all-attributes"<<endl;	   
	
      } 
    
    transformationChild = transformationChild->getNextSibling();
    XMLString::release(&localvalue);
  }
  return;
} /* PolicyDB :: processTransformations(DOMNode *transformations) */


/* PolicyDB :: processServicePermission(char *service_type ,char *service_value)
 * Called when parsing <provide-services> element. Inserts the parsed element pair to the database
 */

void PolicyDB :: processServicePermission(char *service_type ,char* service_value) {
    
    cout<<"Service-type is  "<<service_type<<" and service-value is "<<service_value<<"\n"<<endl;   
    query->insert_servicepermission((char *)policy_rule->presentity,(char *)policy_rule->rule_id,service_type,service_value);
    return;
} /* PolicyDB :: processServicePermission(char *service_type,char *service_value) */

    
/* PolicyDB :: processDevicePermission(char *device_type ,char *device_value)
 * Called when parsing <provide-devices> element. Inserts the parsed element pair to the database
 */

void PolicyDB :: processDevicePermission(char *device_type ,char* device_value) {
    
    cout<<"Device-type is  "<<device_type<<" and device-value is "<<device_value<<"\n"<<endl;   
    query->insert_devicepermission((char *)policy_rule->presentity,(char *)policy_rule->rule_id,device_type,device_value);
    return;       
} /* PolicyDB :: processDevicePermission(char *device_type, char *device_value) */

  
/* PolicyDB :: processPersonPermission(char *person_type ,char *person_value)
 * Called when parsing <provide-persons> element. Inserts the parsed element pair to the database
 */

void PolicyDB :: processPersonPermission(char *person_type ,char* person_value) {
    
    cout<<"Person-type is  "<<person_type<<" and person-value is "<<person_value<<"\n"<<endl;   
    // write code for inserting this pair into person table
    query->insert_personpermission((char *)policy_rule->presentity,(char *)policy_rule->rule_id,person_type,person_value);
    return;        
}/* PolicyDB :: processPersonPermission(char *person_type, char* person_value) */  
            
/* PolicyDB :: convert_to_sqldatetime(string xmldate)
 * Converts datetime from XMLDateTime format to MYSQL DateTime format.
 * For times with zone information (+/-) ,this function generates a string CONVERT_TZ(datetime)
 * so that it can be used as a part of the MYSQL query along with all elements.All conversions
 * are done to UTC time zone.
 */
  
string PolicyDB :: convert_to_sqldatetime (string xmldate) {
 
        string xmldatecpy = xmldate;
        string::size_type pos1 = xmldatecpy.find("T",0);
	string::size_type pos2 = xmldatecpy.find("+",0);
	string::size_type pos3 = xmldatecpy.find("-",pos1);
	string::size_type pos4 = xmldatecpy.find("Z",0);
        string query;
	
	// replacing T with ' '
	if (pos1 != string::npos) {
	   xmldatecpy[pos1]=' ';
	}

	// time is GMT Z
	if(pos4 != string::npos) {
	  xmldatecpy.erase(pos4,1);
	  query = "'"+xmldatecpy+"'";	 
	} 

	// time is with + zone
	else if(pos2 != string::npos) {
	  string localtime = xmldatecpy.substr(0,pos2);
	  string zonetime = xmldatecpy.substr(pos2,xmldatecpy.length()-pos2);
	  query = "CONVERT_TZ('" + localtime+"','" + zonetime +"','+00:00')";
          
        }	
	// time is with - zone 
	else if(pos3 != string::npos) {

	  string localtime1 = xmldatecpy.substr(0,pos3);
	  string zonetime1 = xmldatecpy.substr(pos3,xmldatecpy.length()-pos3);
	  query = "CONVERT_TZ('" + localtime1+"','" + zonetime1 +"','+00:00')";
	       	
	}
	//else  time is WITHOUT zone info
	else {
		xmldatecpy = "'"+ xmldatecpy + "'";
		query = xmldatecpy;
	}
        cout<<"DATE :"<<query<<endl;
        return query;	
} /* PolicyDB::convert_to_sqldatetime(string xmldate) */  


/* PolicyDB :: processRule (DOMNode *rule,const XMLCh *common_policy_prefix)
 * This function parses <rule> and appropriately calls the processConditions,
 * processActions and processTranformations functions
 */

void PolicyDB :: processRule (DOMNode *rule,const XMLCh *common_policy_prefix) {

    // process id attribute of rule(rule-id)
    // now processing children of rule
    
    //modify this
   if(((DOMElement *)rule)->getAttributeNode(XMLString::transcode("id"))!=NULL) {
       policy_rule->rule_id = XMLString::transcode(((DOMElement *)rule)->getAttribute(XMLString::transcode("id")));
       cout<<"Rule id  is "<<policy_rule->rule_id<<"\n"<<endl;
   }
   else {
       cout<<"No id attribute present \n"<<endl;
   }  
 
   DOMNode *element = rule->getFirstChild();

   while(element != NULL) {
       if(strcmp(XMLString::transcode(element->getNodeName()),"#text") == 0) {
	  element = element->getNextSibling();
	  continue;
       }
	
       cout<<"Element encountered : "<<XMLString::transcode(element->getNodeName())<<endl;			       
       char* localvalue = XMLString::transcode(element->getLocalName());
       cout<<"Local value :"<<localvalue<<endl; 	
       char* prefixvalue = XMLString::transcode(element->getPrefix());
       cout<<"Prefix value :"<<prefixvalue<<endl;	
       char* common_prefix = XMLString::transcode(common_policy_prefix);
       cout<<"Common policy prefix :"<<common_prefix<<endl;	

       if((strcmp(localvalue,"conditions")==0) && (strcmp(prefixvalue,common_prefix) == 0)){
           cout<<"entering processCondition function \n"<<endl;
           processConditions(element,common_policy_prefix);
           cout<<"exiting processCondition function \n"<<endl;
       }
       
       else if((strcmp(localvalue,"actions")==0) && (strcmp(prefixvalue,common_prefix) == 0)) {
           cout<<"entering processAction function \n"<<endl;
           processActions(element);
           cout<<"exiting processAction function \n"<<endl;
       }
           
       else if ((strcmp(localvalue,"transformations")==0) && (strcmp(prefixvalue,common_prefix) == 0)){
           cout<<"entering processTransformation function \n"<<endl;
           processTransformations(element);
           cout<<"exiting processTransformation function \n"<<endl;
       }
                  
       element = element->getNextSibling();
       XMLString::release(&localvalue);
       XMLString::release(&prefixvalue);
       XMLString::release(&common_prefix);
  }// end while
  return;
}/* PolicyDB :: processRule (DOMNode *rule,const XMLCh *common_policy_prefix) */

/* PolicyDB :: reset_rule()
 * Resets the rule object to default values
 */

void PolicyDB :: reset_rule() {

   policy_rule->rule_id = NULL;
    // conditions

    // identity

    // <one>
    policy_rule->one_id = NULL;
    policy_rule->one_scheme = NULL;

    // <many>
    policy_rule->except_domain = NULL;
    policy_rule->except_id = NULL;
    policy_rule->many_domain = NULL;
    policy_rule->except_scheme = NULL;


    // other conditions
    policy_rule->sphere = NULL;
    policy_rule->validity_from ="'0000-00-00 00:00:00'"; //NULL;
    policy_rule->validity_to = "'0000-00-00 00:00:00'";//NULL;

    // actions
    policy_rule->sub_handling = "block";
    
    //transformations
    //devices
    policy_rule->all_devices = "false";
    //services
    policy_rule->all_services = "false";
    //persons
    policy_rule->all_persons = "false";

    //other permissions
    policy_rule->provide_activities = "false";
    policy_rule->provide_class = "false";
    policy_rule->provide_device_id="false";
    policy_rule->provide_mood = "false";
    policy_rule->provide_place_is = "false";
    policy_rule->provide_place_type = "false";
    policy_rule->provide_privacy = "false";
    policy_rule->provide_relationship = "false";
    policy_rule->provide_status_icon = "false";
    policy_rule->provide_sphere = "false";
    policy_rule->provide_time_offset = "false";
    policy_rule->provide_user_input = "false";
    policy_rule->provide_note = "false";
    policy_rule->provide_unknown_attribute = "false";
    policy_rule->unknown_attribute_name = NULL;
    return;
}/* PolicyDB :: reset_rule() */


/* PolicyDB :: processRuleset(DOMNode *ruleset,const XMLCh *common_policy_prefix)
 * This function process a ruleset.It inserts each rule of a ruleset in the database.  
 */

void PolicyDB :: processRuleset(DOMNode *ruleset,const XMLCh *common_policy_prefix) {

    DOMNode *rule =  ruleset->getFirstChild();
    

    while (rule != NULL) {
           
      if(strcmp(XMLString::transcode(rule->getNodeName()),"#text") == 0) {
	rule = rule->getNextSibling();
	continue;
      }
  		
      char* localvalue = XMLString::transcode(rule->getLocalName());
      char* prefixvalue = XMLString::transcode(rule->getPrefix());
      char* common_prefix = XMLString::transcode(common_policy_prefix);
    
      cout<<"Element encountered : "<<XMLString::transcode(rule->getNodeName())<<endl;
      if((strcmp(localvalue,"rule")==0) && (strcmp(prefixvalue,common_prefix) == 0)){
	cout<<"Enter processRule "<<endl; 
	processRule(rule,common_policy_prefix);
	cout<<"Exit processRule "<<endl;   

 
        query->insert_ruleset(policy_rule->presentity,policy_rule->rule_id,policy_rule->sphere,(char *)policy_rule->validity_from.c_str(),(char *)policy_rule->validity_to.c_str(),policy_rule->sub_handling,policy_rule->all_services,policy_rule->all_devices,policy_rule->all_persons,policy_rule->provide_activities,policy_rule->provide_class,policy_rule->provide_device_id,policy_rule->provide_mood,policy_rule->provide_place_type,policy_rule->provide_place_is,policy_rule->provide_privacy,policy_rule->provide_relationship,policy_rule->provide_status_icon,policy_rule->provide_sphere,policy_rule->provide_time_offset,policy_rule->provide_user_input,policy_rule->provide_note,policy_rule->provide_unknown_attribute,policy_rule->unknown_attribute_name);

	reset_rule();


      }
        
      rule = rule->getNextSibling();
      XMLString::release(&localvalue);
      XMLString::release(&prefixvalue);
      XMLString::release(&common_prefix);
    }
      
}/* PolicyDB::processRuleset(DOMNode *ruleset,const XMLCh *common_policy_prefix) */ 


