/**
 * SubscriptionAuthorization.cpp: SubscriptionAuthorization.h implementation
 * Copyright 2005 by Columbia University; all rights reserved
 */

#include "SubscriptionAuthorization.h"

/* SubscriptionAuthorization :: SubscriptionAuthorization() 
 * Constructor - Creates a new PolicyQuery Object(query)
 */

SubscriptionAuthorization :: SubscriptionAuthorization() {

    query = new PolicyQuery("test","metro-north.cs.columbia.edu","root","tiger");
    
}/* SubscriptionAuthorization :: SubscriptionAuthorization() */

   
/* SubscriptionAuthorization :: ~SubscriptionAuthorization() 
 * Destructor - Deletes the query object.
 */
 
SubscriptionAuthorization :: ~SubscriptionAuthorization() {

    delete query;
}/* SubscriptionAuthorization :: ~SubscriptionAuthorization() */


/* SubscriptionAuthorization :: getAuthorization(char *presentity, char *watcher, char *domain, char *scheme, char *sphere) 
 * Gets the final subscription handling value for a subscriber of a presentity
 * Returns the integer subscription value
 * Returns -100 if no presentity found and -101  if no suitable rule found
 */

int SubscriptionAuthorization :: getAuthorization(char *presentity, char *watcher, char *domain, char *scheme, char *sphere) {
    
    MYSQL_ROW sub_row;
    MYSQL_RES *pres_res;

    pres_res = query->select_presentity(presentity);	
    if (mysql_fetch_row(pres_res) == NULL) {
	return -100;
    }
	
    list<const char *> ruleid_list;
    getRuleidList(presentity,watcher,domain,scheme,sphere,ruleid_list);
    if (ruleid_list.size() == 0) {
	return -101;
    }
 
    MYSQL_RES *subscription_result = query->getsubscription(convert_list_to_string(ruleid_list),presentity);

    while(((sub_row = mysql_fetch_row(subscription_result)) != NULL) && (mysql_num_rows(subscription_result) == 1)) {
        if(strcmp(sub_row[0],"block") == 0) {
            mysql_free_result(subscription_result);
            return 0;
        }
        else if(strcmp(sub_row[0],"confirm") == 0) {
            mysql_free_result(subscription_result);
            return 1;
        }
        else  if(strcmp(sub_row[0],"polite-block") == 0) {
            mysql_free_result(subscription_result);
            return 2;
        }
        else  if(strcmp(sub_row[0],"allow") ==  0) {
            mysql_free_result(subscription_result);
            return 3;
        }
    }
	
    return -1;
    
}/* SubscriptionAuthorization :: getAuthorization(char *presentity, char *watcher, char *domain, char *scheme, char *sphere) */


/* SubscriptionAuthorization :: convert_list_to_string(list<const char *>ruleid_list) 
 * Converts a list ruleid_list to string form to be used in MYSQL query
 * Returns string format of ruleid_list
 */
string SubscriptionAuthorization :: convert_list_to_string(list<const char *>ruleid_list) {
    
    string ruleid_set = "";
    list<const char *>::iterator ruleid_iterator ;
    if (ruleid_list.size()!=0) {
	for(ruleid_iterator = ruleid_list.begin(); ruleid_iterator != ruleid_list.end(); ++ruleid_iterator) {
        
        ruleid_set += "\'";
        ruleid_set += *ruleid_iterator;
        ruleid_set.append("\'");
        ruleid_set.append(",");
       }
    
 
      // Remove last comma 
      ruleid_set.erase(ruleid_set.size()-1);
      cout<<"List of ruleids string : "<<ruleid_set<<endl;    
    }
    return ruleid_set;
    
}/* SubscriptionAuthorization :: convert_list_to_string(list<const char *>ruleid_list) */


/* SubscriptionAuthorization :: getRuleidList(char *presentity,char *subscriber,char *domain, char *scheme,
 *                              char *sphere, list<const char*> & ruleid_list)
 * Gets a list of all rules that match the subscriber based on identity,sphere and validity(datetime)
 * Returns the final list of matching ruleids 
 */

list<const char *> SubscriptionAuthorization :: getRuleidList(char *presentity,char *subscriber,char *domain, char *scheme,char *sphere, list<const char*> & ruleid_list) {
    
    cout<<"Getting matching ruleid list"<<endl;
    MYSQL_RES *result_identity;
    MYSQL_RES *result_ruleset;
    MYSQL_ROW identity_row;
    MYSQL_ROW ruleset_row;
    //list<const char *> ruleid_list1;
    result_identity = query->select_ruleidentity(presentity,subscriber,domain,scheme);
    
    if(mysql_num_rows(result_identity) == 0) {
	cout << "returning nothing" << endl;
	
	return ruleid_list;	
    }
    		
    while((identity_row = mysql_fetch_row(result_identity))!= NULL) {
        
	
        cout<<"\nSubquerying Ruleid : "<<identity_row[0]<<endl;
         
        result_ruleset = query->select_rulematch(presentity,identity_row[0],sphere);
        while((ruleset_row = mysql_fetch_row(result_ruleset)) != NULL) {
            ruleid_list.push_back(ruleset_row[0]);
        }
        mysql_free_result(result_ruleset);      
            
    }
  

   // print list contents
   cout<<"\nRuleid List Contents:\n"; 
   copy(ruleid_list.begin(),ruleid_list.end(),ostream_iterator<const char *>(cout," ")); 
   cout<<endl;
   mysql_free_result(result_identity);
   return ruleid_list;
                     
}/* SubscriptionAuthorization :: getRuleidList(char *presentity,char *subscriber,char *domain, char *scheme,
                                 char *sphere, list<const char*> & ruleid_list) */




